class_name Rain
extends Node2D


const HEART = preload("res://entity/heart/heart.tscn")

export var amount := 20
export var spawn_pause := 0.25
export var spawn_limit := 3
export(float, 0.01, 65535.0) var mass := 1.0
export var in_control := true

var _hearts := []
var _spawn_elasped := 0.0


func _ready() -> void:
	randomize()
	if in_control:
		_spawn_hearts()


func _process(delta: float) -> void:
	_handle_hearts(delta)


func _spawn_heart() -> void:
	var heart: Heart = HEART.instance()
	add_child(heart)
	if not in_control:
		heart.mode = RigidBody2D.MODE_CHARACTER
	heart.mass = mass
	var size: Vector2 = heart.SPRITE_SIZE
	var h_vec: float = pow(-1, randi() % 2)
	heart.linear_velocity.y *= h_vec
	if in_control:
		heart.position.y = get_viewport().size.y + size.y if h_vec > 0 else -size.y
		var limit_x: float = get_viewport().size.x - size.x * 1.2
		heart.position.x = rand_range(size.x * 1.2, limit_x)
	else:
		heart.position.y = (get_viewport().size.y / 2 + size.y) * h_vec
		var limit_x: float = get_viewport().size.x / 2 - size.x * 1.2
		heart.position.x = rand_range(-limit_x, limit_x)
	_hearts.append(heart)


func _spawn_hearts() -> void:
	var limit: int = amount - _hearts.size()
	if limit <= 0:
		return
	if limit > spawn_limit:
		limit = spawn_limit
	for _i in limit:
		_spawn_heart()


func _remove_heart(heart: Heart) -> void:
	call_deferred("remove_child", heart)
	_hearts.erase(heart)
	heart.queue_free()


func _handle_hearts(delta: float) -> void:
	var size: Vector2 = Heart.SPRITE_SIZE
	var for_remove := []
	var limit_x: float
	var limit_y: float
	if in_control:
		limit_x = get_viewport().size.x + size.x
		limit_y = get_viewport().size.y + size.y * 1.5
	else:
		limit_x = get_viewport().size.x / 2 + size.x
		limit_y = get_viewport().size.y / 2 + size.y * 1.5
	for heart in _hearts:
		if in_control:
			if (
					heart.position.x > limit_x or heart.position.x < -size.x
					or heart.position.y > limit_y or heart.position.y < -limit_y * 1.5
			):
				for_remove.append(heart)
		else:
			if (
					heart.position.x > limit_x or heart.position.x < -limit_x
					or heart.position.y > limit_y or heart.position.y < -limit_y
					or abs(heart.linear_velocity.y) < 1
			):
				for_remove.append(heart)
	for heart in for_remove:
		_remove_heart(heart)
	_spawn_elasped += delta
	if _spawn_elasped >= spawn_pause:
		_spawn_elasped = 0.0
		_spawn_hearts()
