class_name Stars
extends Node2D


const COLORS := [
	Color.white,
	Color.whitesmoke,
	Color.aliceblue,
	Color.aqua,
	Color.aquamarine,
	Color.azure,
	Color.cyan,
	Color.chartreuse,
	Color.floralwhite,
	Color.ghostwhite,
	Color.lime,
	Color.orange,
]
const SQUARE_AREA := 10000

export var size_min := 0.1
export var size_max := 1.0
export var speed_min := 50.0
export var speed_max := 100.0
export var density := 1.0
export var amount_limit := 300
export var static_stars := 0
export var in_conrol := false

var _stars := []
var _random := RandomNumberGenerator.new()


func _init() -> void:
	_random.randomize()


func _ready() -> void:
	get_tree().get_root().connect("size_changed", self, "_resize")
	_resize()


func _process(delta: float) -> void:
	if not visible or not get_parent().visible:
		return
	if not Global.store.get_property(Global.Store.Property.STARS_MOVE):
		return
	var size: Vector2 = Global.current_scene.get_viewport().size
	var for_remove := []
	for star in _stars:
		var limit_y: float = -(size.y / 2 - star.size)
		if not star.static:
			star.y = star.y - star.speed * delta
		if star.y < limit_y:
			var new_star := _make_random_star(size, star.static)
			new_star.y = size.y / 2 + new_star.size
			for_remove.append(star)
			_stars.append(new_star)
	for star in for_remove:
		_stars.erase(star)
	update()


func _draw() -> void:
	var x: float = (Global.current_scene.get_viewport().size.x / 2) if in_conrol else 0.0
	var y: float = (Global.current_scene.get_viewport().size.y / 2) if in_conrol else 0.0
	for star in _stars:
		draw_circle(Vector2(star.x + x, star.y + y), star.size, star.color)


func _resize() -> void:
	_fill_stars()
	update()


func _fill_stars() -> void:
	var size: Vector2 = Global.current_scene.get_viewport().size
	var amount := _calc_amount(size)
	var static_counter := static_stars
	_stars.clear()
	for i in amount:
		var is_static: bool = true if static_counter > 0 else false
		var star := _make_random_star(size, is_static)
		_stars.append(star)
		static_counter -= 1 if static_counter > 0 else 0


func _calc_amount(size: Vector2) -> int:
	var area: float = size.x * size.y
	var amount: int = int(area / SQUARE_AREA * density)
	if amount > amount_limit:
		amount = amount_limit
	return amount


func _make_random_star(size: Vector2, is_static: bool) -> Dictionary:
	var size_star: float = _random.randf_range(size_min, size_max)
	var limit_x: float = size.x / 2 - size_star
	var limit_y: float = size.y / 2 - size_star
	var star := {
		"x": _random.randf_range(-limit_x, limit_x),
		"y": _random.randf_range(-limit_y, limit_y),
		"size": size_star,
		"color": COLORS[_random.randi_range(0, COLORS.size() - 1)],
		"speed": _random.randf_range(speed_min, speed_max),
		"static": is_static,
	}
	return star
