class_name Bullet
extends RigidBody2D


signal on_die

const Variants = preload("res://entity/bullet/variants.gd")
const HP = preload("res://entity/hp.gd")

const _LIGHT_BANG_ENERGY := 1.3
const _SLOW_LIMIT := 10.0

var hp: HP = HP.new()

var _impulse := 1000.0
var _damage: float = 1.0

onready var _shape: CollisionShape2D = $CollisionShape2D
onready var _current_sprite: AnimatedSprite = $AnimatedSprite


func _ready() -> void:
	hp.connect("hp_change", self, "_on_hp_change")
	if Global.store.get_property(Global.Store.Property.LIGHT):
		var light: Light2D = load("res://entity/bullet/light.tscn").instance()
		add_child(light)


func _notification(what) -> void:
	if what != NOTIFICATION_PREDELETE:
		return
	hp.free()


func set_variant(variant) -> void:
	var variants: Variants = Variants.new()
	var vars: Dictionary = variants.get_vars(variant)
	_impulse = vars.impulse_value
	_damage = vars.damage
	hp.set_max(vars.hp)
	hp.set_hp(vars.hp)
	mass = vars.mass
	#_set_animation(vars.animation)
	call_deferred("_set_animation", vars.animation)
	variants.free()


func get_impulse_value() -> float:
	return _impulse


func get_sprite_size() -> Vector2:
	return _current_sprite.frames.get_frame("default", 0).get_size()


func get_radius() -> float:
	var size: Vector2 = get_sprite_size()
	var radius: float = size.x / 2 if size.x > size.y else size.y / 2
	return radius


func get_damage() -> float:
	return _damage


func is_active() -> bool:
	if _current_sprite.animation == "default":
		return true
	return false


func bang() -> void:
	if _current_sprite.animation == "bang":
		return
	_shape.set_deferred("disabled", true)
	_current_sprite.play("bang")
	_current_sprite.connect("animation_finished", self, "_on_timeout")
	var light = get_node_or_null("Light2D")
	if light:
		light.energy = _LIGHT_BANG_ENERGY


func _on_hp_change(new_hp: float, _old_hp: float) -> void:
	if new_hp == 0:
		bang()


func _on_timeout() -> void:
	emit_signal("on_die", self)


func _set_animation(name: String) -> void:
	var frames: SpriteFrames = _current_sprite.frames
	frames = load("res://entity/bullet/anim/" + name + ".tres")
	_current_sprite.frames = frames
	var radius := int(round(get_radius()))
	_shape.shape.radius = radius


func _on_Bullet_body_exited(_body: Node) -> void:
	var abs_velocity := abs(linear_velocity.x) + abs(linear_velocity.y)
	if abs_velocity < _SLOW_LIMIT:
		bang()
