class_name Drop
extends RigidBody2D


signal on_die

enum DropVariant {
	YELLOW,
	RED,
	BLUE,
	NONE,
}

const HP = preload("res://entity/hp.gd")

const _HP_YELLOW := 1.0
const _HP_RED := 2.0
const _HP_BLUE := 3.0
const _DAMAGE_YELLOW := 2.0
const _DAMAGE_RED := 4.0
const _DAMAGE_BLUE := 8.0

var hp: HP = HP.new()

var _variant = DropVariant.YELLOW
var _damage: float = _DAMAGE_YELLOW

onready var _sprite: AnimatedSprite = $AnimatedSprite


func _ready() -> void:
	hp.connect("hp_change", self, "_on_hp_change")
	match(_variant):
		DropVariant.YELLOW:
			_sprite.play("yellow")
			hp.set_max(_HP_YELLOW)
			hp.set_hp(_HP_YELLOW)
		DropVariant.RED:
			_sprite.play("red")
			hp.set_max(_HP_RED)
			hp.set_hp(_HP_RED)
		DropVariant.BLUE:
			_sprite.play("blue")
			hp.set_max(_HP_BLUE)
			hp.set_hp(_HP_BLUE)


func _notification(what) -> void:
	if what != NOTIFICATION_PREDELETE:
		return
	hp.free()


func set_variant(drop_variant) -> void:
	_variant = drop_variant
	match(drop_variant):
		DropVariant.YELLOW:
			_damage = _DAMAGE_YELLOW
		DropVariant.RED:
			_damage = _DAMAGE_RED
		DropVariant.BLUE:
			_damage = _DAMAGE_BLUE


func get_sprite_size() -> Vector2:
	return Vector2(5, 20)


func get_damage() -> float:
	return _damage


func get_variant():
	return _variant


func get_string_variant() -> String:
	match(_variant):
		DropVariant.YELLOW:
			return "yellow"
		DropVariant.RED:
			return "red"
		DropVariant.BLUE:
			return "blue"
		_:
			return "none"


func _on_hp_change(new_hp: float, _old_hp: float) -> void:
	if new_hp == 0:
		emit_signal("on_die", self)
