class_name Fragment
extends RigidBody2D


const Variants = preload("res://entity/fragment/variants.gd")

var _impulse_min := 200.0
var _impulse_max := 1000.0

onready var _current_sprite: Sprite = $Sprite
onready var _particles: CPUParticles2D = $CPUParticles2D


func _ready() -> void:
	randomize()
	rotate(rand_range(0, PI * 2))
	if not Global.store.get_property(Global.Store.Property.PARTICLES):
		_particles.emitting = false


func set_variant(variant) -> void:
	var variants: Variants = Variants.new()
	var vars: Dictionary = variants.get_vars(variant)
	_impulse_min = vars.impulse_min
	_impulse_max = vars.impulse_max
	_set_texture(vars.texture)
	variants.free()


func get_impulse_range() -> Dictionary:
	return {"min": _impulse_min, "max": _impulse_max}


func get_sprite_size() -> Vector2:
	return _current_sprite.texture.get_size()


func _set_texture(name: String) -> void:
	var texture: Texture = _current_sprite.texture
	match name:
		"standard":
			texture = load("res://entity/fragment/png/fragment.png")
		"st_1":
			texture = load("res://entity/fragment/png/fragment1.png")
		"st_2":
			texture = load("res://entity/fragment/png/fragment2.png")
		"st_3":
			texture = load("res://entity/fragment/png/fragment3.png")
		"st_4":
			texture = load("res://entity/fragment/png/fragment4.png")
		"st_5":
			texture = load("res://entity/fragment/png/fragment5.png")
		"st_6":
			texture = load("res://entity/fragment/png/fragment6.png")
		"st_7":
			texture = load("res://entity/fragment/png/fragment7.png")
		"st_8":
			texture = load("res://entity/fragment/png/fragment8.png")
		"gun":
			texture = load("res://entity/fragment/png/fragment-gun.png")
		"gun_flat":
			texture = load("res://entity/fragment/png/fragment-gun1.png")
		"gun_red":
			texture = load("res://entity/fragment/png/fragment-gun2.png")
		"gun_blue":
			texture = load("res://entity/fragment/png/fragment-gun3.png")
	_current_sprite.texture = texture
