class_name Gift
extends RigidBody2D


signal on_end_play
signal on_collide
signal fling_end

enum GiftVariant {
	BORDER,
	DROP,
	FLING,
	RANDOM,
	RENEWAL,
	REPAIR,
	BOOST,
}

const SPRITE_SIZE := Vector2(30, 30)

const _END_SHAPE_SCALE := 25.0
const _SCALE_SPEED := 21.0
const _FLING_LAYER := 1
const _NORMAL_LAYER := 3

var _variant = GiftVariant.BORDER
var _fling_active := false
var _mark_remove := false

onready var _sprite: Sprite = $Sprite
onready var _shape: CollisionShape2D = $CollisionShape2D
onready var _sound_activate: AudioStreamPlayer2D = $SoundActivate


func _ready() -> void:
	match(_variant):
		GiftVariant.BORDER:
			_sprite.texture = load("res://entity/gift/png/gift-border.png")
		GiftVariant.DROP:
			_sprite.texture = load("res://entity/gift/png/gift-drop.png")
		GiftVariant.FLING:
			_sprite.texture = load("res://entity/gift/png/gift-fling.png")
		GiftVariant.RANDOM:
			_sprite.texture = load("res://entity/gift/png/gift-random.png")
		GiftVariant.RENEWAL:
			_sprite.texture = load("res://entity/gift/png/gift-renewal.png")
		GiftVariant.REPAIR:
			_sprite.texture = load("res://entity/gift/png/gift-repair.png")
		GiftVariant.BOOST:
			_sprite.texture = load("res://entity/gift/png/gift-x.png")
	_sound_activate.connect("finished", self, "_end_play")


func _process(delta: float) -> void:
	if _fling_active:
		if _shape.scale.x == _END_SHAPE_SCALE:
			_shape.scale.x = 1
			_shape.scale.y = 1
			collision_layer = _NORMAL_LAYER
			_fling_active = false
			emit_signal("fling_end", self)
		_shape.scale.x += _SCALE_SPEED * delta
		_shape.scale.y += _SCALE_SPEED * delta
		if _shape.scale.x >= _END_SHAPE_SCALE:
			_shape.scale.x = _END_SHAPE_SCALE
			_shape.scale.y = _END_SHAPE_SCALE


func set_variant(variant) -> void:
	_variant = variant


func get_variant():
	return _variant


func fling() -> void:
	if _fling_active:
		return
	_fling_active = true
	set_deferred("collision_layer", _FLING_LAYER)


func mark_remove() -> void:
	_mark_remove = true
	_shape.set_deferred("disabled", true)
	modulate = "#7fffffff"


func _end_play() -> void:
	if _mark_remove:
		emit_signal("on_end_play", self)


func _on_Gift_body_entered(body: Node) -> void:
	if not body.is_in_group("bullet"):
		return
	_sound_activate.play()
	emit_signal("on_collide", self, body)
