class_name Meteor
extends RigidBody2D


signal on_collide
signal on_die
signal on_fragments

const Variants = preload("res://entity/meteor/variants.gd")
const HP = preload("res://entity/hp.gd")

var hp: HP = HP.new()

var _rotate_impulse_min := 0.0
var _rotate_impulse_max := 2000.0
var _impulse_min := 10.0
var _impulse_max := 100.0
var _fragments: Array
var _damage := 1.0
var _current_variant

onready var _current_sprite: AnimatedSprite = $AnimatedSprite
onready var _shape: CollisionShape2D = $CollisionShape2D
onready var _explosion: AudioStreamPlayer2D = $Explosion


func _ready() -> void:
	_current_sprite.connect("animation_finished", self, "_on_timeout")
	hp.connect("hp_change", self, "_on_hp_change")
	randomize()
	var rotation_impulse: float = rand_range(
			_rotate_impulse_min,
			_rotate_impulse_max
	)
	rotation_impulse *= pow(-1, randi() % 2)
	# crutch for bug https://github.com/godotengine/godot/issues/29888
	yield(get_tree(), "idle_frame")
	call_deferred("apply_torque_impulse", rotation_impulse)


func _notification(what) -> void:
	if what != NOTIFICATION_PREDELETE:
		return
	hp.free()


func set_variant(variant) -> void:
	var variants: Variants = Variants.new()
	var vars: Dictionary = variants.get_vars(variant)
	_rotate_impulse_min = vars.rotate_impulse_min
	_rotate_impulse_max = vars.rotate_impulse_max
	_impulse_min = vars.impulse_min
	_impulse_max = vars.impulse_max
	_fragments = vars.fragments
	_damage = vars.damage
	hp.set_max(vars.hp)
	hp.set_hp(vars.hp)
	mass = vars.mass
	_set_animation(vars.animation)
	_current_variant = variant
	variants.free()


func bang() -> void:
	_shape.set_deferred("disabled", true)
	_current_sprite.play("bang")
	_explosion.play()


func get_radius() -> float:
	var size: Vector2 = get_sprite_size()
	var radius: float = size.x / 2 if size.x > size.y else size.y / 2
	return radius


func get_impulse_range() -> Dictionary:
	return {"min": _impulse_min, "max": _impulse_max}


func get_sprite_size() -> Vector2:
	return _current_sprite.frames.get_frame("default", 0).get_size()


func get_damage() -> float:
	return _damage


func get_variant():
	return _current_variant


func _set_animation(name: String) -> void:
	var frames: SpriteFrames = _current_sprite.frames
	frames = load("res://entity/meteor/anim/" + name + ".tres")
	_current_sprite.frames = frames
	var radius := int(round(get_radius()))
	_shape.shape.radius = radius


func _on_hp_change(new_hp: float, _old_hp: float) -> void:
	if new_hp == 0:
		bang()


func _on_timeout() -> void:
	for data in _fragments:
		emit_signal("on_fragments", {
				"position": position,
				"radius": get_radius(),
				"fragments_range": data,
				"fragments_variant": data.variant,
		})
	emit_signal("on_die", self)


func _on_Meteor_body_entered(body: Node) -> void:
	emit_signal("on_collide", self, body)
