class_name ActionArea
extends Area2D


signal action_activate

export var action_name: String
export var animation_name: String
export var x_offset: int = 0
export var y_offset: int = 60

var _enabled := true

onready var _current_sprite: AnimatedSprite = $AnimatedSprite
onready var _sound_activate: AudioStreamPlayer2D = $SoundActivate


func _ready() -> void:
	get_tree().get_root().connect("size_changed", self, "_resize")
	_resize()
	_current_sprite.animation = animation_name


func set_speed(speed: int) -> void:
	_current_sprite.frames.set_animation_speed(animation_name, speed)


func set_animation_name(name: String) -> void:
	animation_name = name
	_current_sprite.animation = animation_name


func reset() -> void:
	_current_sprite.stop()
	_current_sprite.frame = 0


func set_enabled(enabled: bool) -> void:
	_enabled = enabled


func _resize() -> void:
	if is_inside_tree():
		position.x = x_offset
		position.y = -(get_viewport().size.y / 2 - y_offset)


func _activate() -> bool:
	if _current_sprite.is_playing():
		return false
	_current_sprite.play(animation_name)
	_sound_activate.play()
	return true


func _on_RandomArea_input_event(_viewport, event, _shape_idx) -> void:
	if _enabled and event is InputEventMouseButton and event.is_pressed() and _activate():
		emit_signal("action_activate", action_name)


func _on_AnimatedSprite_animation_finished() -> void:
	reset()
