class_name Actions
extends Node2D


enum ActionType {
	RANDOM,
	BOOST,
	BORDER,
	DROP,
}

onready var _action_random: ActionArea = $RandomArea
onready var _boost: ActionArea = $XArea
onready var _border: ActionArea = $BorderArea
onready var _drop: ActionArea = $DropArea


func setup_game(game) -> void:
	_action_random.connect("action_activate", game, "action_activate")
	_boost.connect("action_activate", game, "action_activate")
	_border.connect("action_activate", game, "action_activate")
	_drop.connect("action_activate", game, "action_activate")


func hide_all() -> void:
	_action_random.hide()
	_boost.hide()
	_border.hide()
	_drop.hide()


func set_action_visible(action, is_visible: bool) -> void:
	match(action):
		ActionType.RANDOM:
			_action_random.visible = is_visible
		ActionType.BOOST:
			_boost.visible = is_visible
		ActionType.BORDER:
			_border.visible = is_visible
		ActionType.DROP:
			_drop.visible = is_visible


func set_animation_speed(action, speed: int) -> void:
	match(action):
		ActionType.RANDOM:
			_action_random.set_speed(speed)
		ActionType.BOOST:
			_boost.set_speed(speed)
		ActionType.BORDER:
			_border.set_speed(speed)
		ActionType.DROP:
			_drop.set_speed(speed)


func set_boost_animation(boost: int) -> void:
	if boost < 2 or boost > 5:
		return
	_boost.set_animation_name("x_" + str(boost))


func set_drop_animation(drop_variant) -> void:
	match(drop_variant):
		Drop.DropVariant.YELLOW:
			_drop.set_animation_name("drop")
		Drop.DropVariant.RED:
			_drop.set_animation_name("drop_1")
		Drop.DropVariant.BLUE:
			_drop.set_animation_name("drop_2")


func set_enabled(action, enabled: bool) -> void:
	match(action):
		ActionType.RANDOM:
			_action_random.set_enabled(enabled)
		ActionType.BORDER:
			_border.set_enabled(enabled)
		ActionType.BOOST:
			_boost.set_enabled(enabled)
		ActionType.DROP:
			_drop.set_enabled(enabled)


func reset() -> void:
	_action_random.reset()
	_boost.reset()
	_border.reset()
	_drop.reset()
