class_name Border
extends StaticBody2D


signal on_deactivate

var _is_ready := false

onready var _sprite: AnimatedSprite = $BorderSprite
onready var _shape: CollisionShape2D = $CollisionShape2D
onready var _timer: Timer = $Timer


func _ready() -> void:
	_is_ready = true


func _exit_tree() -> void:
	_timer.paused = true


func _enter_tree() -> void:
	if not _is_ready:
		return
	_timer.paused = false


func activate_border(timeout: float) -> void:
	if visible:
		return
	show()
	_shape.set_deferred("disabled", false)
	_sprite.play("start")
	_sprite.connect("animation_finished", self, "_start_finished")
	_timer.start(timeout)


func deactivate_border() -> void:
	if not visible:
		return
	_timer.stop()
	_sprite.play("stop")
	_sprite.connect("animation_finished", self, "_stop_finished")
	emit_signal("on_deactivate", Actions.ActionType.BORDER)


func force_deactivate_border() -> void:
	if not visible:
		return
	_timer.stop()
	_shape.set_deferred("disabled", true)
	hide()
	emit_signal("on_deactivate", Actions.ActionType.BORDER)


func _start_finished() -> void:
	_sprite.disconnect("animation_finished", self, "_start_finished")
	_sprite.play("active")


func _stop_finished() -> void:
	_sprite.disconnect("animation_finished", self, "_stop_finished")
	_shape.set_deferred("disabled", true)
	hide()
