class_name Bullets
extends Node2D


const BULLET = preload("res://entity/bullet/bullet.tscn")
const Postponer = preload("res://game/gd/postponer.gd")
const BulletsCache = preload("res://game/gd/bullets_cache.gd")

var _postponer: Postponer = Postponer.new()
var _cache: BulletsCache = BulletsCache.new()
var _bullets := []


func _ready() -> void:
	_cache.fill()


func _process(_delta: float) -> void:
	if _postponer.tick():
		_handle_bullets()
		_cache.fill()


func _notification(what) -> void:
	if what != NOTIFICATION_PREDELETE:
		return
	_postponer.free()
	_cache.wait_thread()
	_cache.free()


func spawn_bullet(data: Dictionary) -> void:
	var bullet: Bullet = _cache.pop()
	add_child(bullet)
	bullet.set_variant(data.variant)
	var gun_radius: float = data.radius
	var bullet_radius: float = bullet.get_radius()
	var radius: float = gun_radius + bullet_radius
	var impulse_direction := Vector2(
			cos(data.rotation) * bullet.get_impulse_value(),
			sin(data.rotation) * bullet.get_impulse_value()
	)
	var spawn_position: Vector2 = data.position + Vector2(radius, 0).rotated(data.rotation)
	bullet.position = spawn_position
	bullet.show()
	bullet.apply_central_impulse(impulse_direction)
	bullet.connect("on_die", self, "_on_bullet_die")
	_bullets.append(bullet)


func clear() -> void:
	for bullet in _bullets:
		call_deferred("remove_child", bullet)
		bullet.queue_free()
	_bullets.clear()
	_cache.wait_thread()
	_cache.clear()


func _handle_bullets() -> void:
	var for_remove := []
	for bullet in _bullets:
		var size: Vector2 = bullet.get_sprite_size()
		var limit_x: float = get_viewport().size.x / 2 + size.x
		var limit_y: float = get_viewport().size.y / 2 + size.y
		if (
				bullet.position.x > limit_x or bullet.position.x < -limit_x
				or bullet.position.y > limit_y or bullet.position.y < -limit_y
		):
			for_remove.append(bullet)
	for bullet in for_remove:
		_remove_bullet(bullet)


func _remove_bullet(bullet: Bullet) -> void:
	call_deferred("remove_child", bullet)
	_bullets.erase(bullet)
	bullet.queue_free()


func _on_bullet_die(bullet: Bullet) -> void:
	_remove_bullet(bullet)
