class_name Fragments
extends Node2D


const FRAGMENT = preload("res://entity/fragment/fragment.tscn")
const Postponer = preload("res://game/gd/postponer.gd")

var _random := RandomNumberGenerator.new()
var _postponer: Postponer = Postponer.new()
var _fragments := []


func _ready() -> void:
	_random.randomize()


func _process(_delta: float) -> void:
	if _postponer.tick():
		_handle_fragments()


func _notification(what) -> void:
	if what != NOTIFICATION_PREDELETE:
		return
	_postponer.free()


func spawn_fragments(data: Dictionary) -> void:
	call_deferred("_deffered_spawn_fragments", data)


func _spawn_fragment(point: Vector2, radius: float, variant) -> void:
	var fragment: Fragment = FRAGMENT.instance()
	add_child(fragment)
	fragment.set_variant(variant)
	var x: float = _random.randf_range(0, radius) * pow(-1, _random.randi() % 2)
	var y: float = _random.randf_range(0, radius) * pow(-1, _random.randi() % 2)
	var pos := Vector2(point.x + x, point.y + y)
	var angle: float = _random.randf_range(0, PI * 2)
	var impulse_range: Dictionary = fragment.get_impulse_range()
	var impulse: float = _random.randf_range(impulse_range.min, impulse_range.max)
	var impulse_direction := Vector2(
			cos(angle) * impulse,
			sin(angle) * impulse
	)
	fragment.position = pos
	fragment.apply_central_impulse(impulse_direction)
	_fragments.append(fragment)


func _deffered_spawn_fragments(data: Dictionary) -> void:
	if not Global.store.get_property(Global.Store.Property.FRAGMENTS_SPAWN):
		return
	var amount: int = _random.randi_range(data.fragments_range.min, data.fragments_range.max)
	for i in amount:
		_spawn_fragment(data.position, data.radius, data.fragments_variant)


func _remove_fragment(fragment: RigidBody2D) -> void:
	call_deferred("remove_child", fragment)
	_fragments.erase(fragment)
	fragment.queue_free()


func _handle_fragments() -> void:
	var for_remove := []
	for fragment in _fragments:
		var size: Vector2 = fragment.get_sprite_size()
		var limit_x: float = get_viewport().size.x / 2 + size.x
		var limit_y: float = get_viewport().size.y / 2 + size.y
		if (
				fragment.position.x > limit_x or fragment.position.x < -limit_x
				or fragment.position.y > limit_y or fragment.position.y < -limit_y
		):
			for_remove.append(fragment)
	for fragment in for_remove:
		_remove_fragment(fragment)
