class_name Gifts
extends Node2D


signal gift_activate

const GIFT = preload("res://entity/gift/gift.tscn")
const Postponer = preload("res://game/gd/postponer.gd")

var _postponer: Postponer = Postponer.new()
var _gifts := []


func _ready() -> void:
	randomize()


func _process(_delta: float) -> void:
	if _postponer.tick():
		_handle_gifts()


func _notification(what) -> void:
	if what != NOTIFICATION_PREDELETE:
		return
	_postponer.free()


func spawn_gift(variant) -> void:
	var gift: Gift = GIFT.instance()
	gift.set_variant(variant)
	var size: Vector2 = gift.SPRITE_SIZE
	gift.position.y = get_viewport().size.y / 2 + size.y
	var limit_x: float = get_viewport().size.x / 2 - get_viewport().size.x / 4
	gift.position.x = rand_range(-limit_x, limit_x)
	gift.connect("on_collide", self, "_gift_collide")
	gift.connect("fling_end", self, "_fling_end")
	gift.connect("on_end_play", self, "_remove_gift")
	_gifts.append(gift)
	call_deferred("add_child", gift)
	Global.counter.increase(GameCounter.CountType.GIFTS_SPAWNED, 1)


func clear() -> void:
	for gift in _gifts:
		call_deferred("remove_child", gift)
		gift.queue_free()
	_gifts.clear()


func _gift_collide(gift: Gift, to: Node) -> void:
	emit_signal("gift_activate", gift.get_variant(), to)
	if gift.get_variant() == Gift.GiftVariant.FLING:
		gift.fling()
	else:
		gift.mark_remove()


func _fling_end(gift: Gift) -> void:
	_remove_gift(gift)


func _remove_gift(gift: Gift) -> void:
	call_deferred("remove_child", gift)
	_gifts.erase(gift)
	gift.queue_free()


func _handle_gifts() -> void:
	var for_remove := []
	var size: Vector2 = Gift.SPRITE_SIZE
	var limit_x: float = get_viewport().size.x / 2 + size.x
	var limit_y: float = get_viewport().size.y / 2 + size.y * 1.5
	for gift in _gifts:
		if (
				gift.position.x > limit_x or gift.position.x < -limit_x
				or gift.position.y > limit_y or gift.position.y < -limit_y
		):
			for_remove.append(gift)
	for gift in for_remove:
		_remove_gift(gift)
