extends Object


enum DonateVariant {
	CRYPTO,
	LINK,
}

const _CONFIG_PATH := "user://asteroids_donate.conf"

var _currencies := [ { "name": "BTC", "addr": "none" } ]
var _link := RestAPI.DONATE_LINK
var _variant = DonateVariant.CRYPTO
var _show_button := false


func _init() -> void:
	_load_donate()
	if _show_button:
		show_donate_button()


func show_donate_button() -> void:
	if _show_button or not Global.store.get_property(Global.Store.Property.NETWORK_REQUESTS):
		return
	var result = Global.http_request(RestAPI.DONATE_ENDPOINT)
	if result != OK:
		push_error("HTTPRequest error: " + str(result))


func can_show_donate_button() -> bool:
	return _show_button


func get_variant():
	return _variant


func get_link() -> String:
	return _link


func get_currencies() -> Array:
	return _currencies


func request_completed(
		result: int, response_code: int, _headers: PoolStringArray, body: PoolByteArray
) -> void:
	var response = null if body.empty() else parse_json(body.get_string_from_utf8())
	if (
			result != HTTPRequest.RESULT_SUCCESS
			or response_code != HTTPClient.RESPONSE_OK
			or typeof(response) != TYPE_DICTIONARY
			or response.endpoint != "donate"
	):
		print_debug("bad API donate response")
		return
	_show_button = response.enabled
	if not response.enabled:
		_save_donate()
		return
	match(response.variant):
		"LINK":
			_variant = DonateVariant.LINK
		"CRYPTO":
			_variant = DonateVariant.CRYPTO
	if _variant == DonateVariant.LINK:
		_link = response.link
	elif _variant == DonateVariant.CRYPTO:
		_currencies = response.currencies
	_save_donate()


func _save_donate() -> void:
	var config = ConfigFile.new()
	config.set_value("donate", "show", true)
	config.set_value("donate", "variant", _variant)
	config.set_value("donate", "link", _link)
	config.set_value("donate", "currencies", _currencies)
	config.save(_CONFIG_PATH)


func _load_donate() -> void:
	var config = ConfigFile.new()
	if config.load(_CONFIG_PATH) != OK:
		return
	_show_button = config.get_value("donate", "show", false)
	_variant = config.get_value("donate", "variant", DonateVariant.LINK)
	_link = config.get_value("donate", "link", "")
	_currencies = config.get_value("donate", "currencies", [ { "name": "BTC", "addr": "none" } ])
