extends Object


signal api_reply

const _CONFIG_PATH := "user://asteroids_review.conf"

var _review_link := RestAPI.REVIEW_LINK
var _show_button := false
var _sent := false


func _init() -> void:
	_sent = _load_is_sent()


func show_review_button() -> void:
	if (
		_show_button
		or _sent
		or not Global.store.get_property(Global.Store.Property.NETWORK_REQUESTS)
	):
		return
	var result = Global.http_request(RestAPI.REVIEW_GET_ENDPOINT)
	if result != OK:
		push_error("HTTPRequest error: " + str(result))


func can_show_review_button() -> bool:
	return _show_button and not _sent


func is_sent() -> bool:
	return _sent


func get_link() -> String:
	return _review_link


func request_completed(
		result: int, response_code: int, _headers: PoolStringArray, body: PoolByteArray
) -> void:
	var response = null if body.empty() else parse_json(body.get_string_from_utf8())
	if (
			result != HTTPRequest.RESULT_SUCCESS
			or response_code != HTTPClient.RESPONSE_OK
			or typeof(response) != TYPE_DICTIONARY
			or (response.endpoint != "review" and response.endpoint != "postreview")
	):
		emit_signal("api_reply", { "error": true })
		print_debug("bad API review response")
		return
	_show_button = response.enabled
	_review_link = response.link
	if response.endpoint == "review":
		return
	if response.success:
		_set_sended()
		emit_signal("api_reply", { "error": false })
	else:
		print(response.error)
		emit_signal("api_reply", { "error": true })


func _set_sended() -> void:
	_sent = true
	var config = ConfigFile.new()
	config.set_value("review", "sent", true)
	config.save(_CONFIG_PATH)


func _load_is_sent() -> bool:
	var config = ConfigFile.new()
	if config.load(_CONFIG_PATH) != OK:
		return false
	var conf_sent: bool = config.get_value("review", "sent", false)
	return conf_sent
