extends Object


enum Property {
	VOLUME_MASTER,
	VOLUME_ENV,
	VOLUME_MUSIC,
	VOLUME_ALERTS,
	STARS_MOVE,
	FRAGMENTS_SPAWN,
	LIGHT,
	PARTICLES,
	NETWORK_REQUESTS,
	LOCALE,
}

const _PATH := "user://asteroids_settings.conf"

var _store := {}  # don't change directly! use `set_property`
var _master_idx: int = AudioServer.get_bus_index("Master")
var _env_idx: int = AudioServer.get_bus_index("Env")
var _music_idx: int = AudioServer.get_bus_index("Music")
var _alerts_idx: int = AudioServer.get_bus_index("Alerts")


func fill_default() -> void:
	#set_property(Property.VOLUME_MASTER, AudioServer.get_bus_volume_db(_master_idx))
	#set_property(Property.VOLUME_ENV, AudioServer.get_bus_volume_db(_env_idx))
	#set_property(Property.VOLUME_MUSIC, AudioServer.get_bus_volume_db(_music_idx))
	#set_property(Property.VOLUME_ALERTS, AudioServer.get_bus_volume_db(_alerts_idx))
	set_property(Property.VOLUME_MASTER, 0.0)
	set_property(Property.VOLUME_ENV, -8.63597)
	set_property(Property.VOLUME_MUSIC, -1.11035)
	set_property(Property.VOLUME_ALERTS, 0.0)
	set_property(Property.STARS_MOVE, true)
	set_property(Property.FRAGMENTS_SPAWN, true)
	set_property(Property.LIGHT, false)
	set_property(Property.PARTICLES, true)
	set_property(Property.NETWORK_REQUESTS, true)
	set_property(Property.LOCALE, TranslationServer.get_locale())


func set_property(key, value) -> void:
	match(key):
		Property.VOLUME_MASTER:
			AudioServer.set_bus_volume_db(_master_idx, value)
		Property.VOLUME_ENV:
			AudioServer.set_bus_volume_db(_env_idx, value)
		Property.VOLUME_MUSIC:
			AudioServer.set_bus_volume_db(_music_idx, value)
		Property.VOLUME_ALERTS:
			AudioServer.set_bus_volume_db(_alerts_idx, value)
		Property.LOCALE:
			TranslationServer.set_locale(value)
	_store[key] = value


func get_property(key):
	return _store[key]


func save_settings() -> void:
	var config = ConfigFile.new()
	for key in _store:
		config.set_value("settings", Property.keys()[key], _store[key])
	config.save(_PATH)


func load_settings() -> void:
	fill_default()
	var config = ConfigFile.new()
	if config.load(_PATH) != OK:
		return
	for key in _store:
		var value = config.get_value("settings", Property.keys()[key], _store[key])
		set_property(key, value)
