class_name RestAPI
extends HTTPRequest


# REST API:
# GET api.example.com/review
# POST api.example.com/postreview
# reply body include:
# - endpoint: "review" or "postreview"
# - enabled: bool
# - link: url
# needed only for postreview:
# - success: bool
# - error: string
#
# POST api.example.com/postreview
# request body include:
# - username: string
# - rating: float (0-1)
# - text: string
# - link: string
#
# GET api.example.com/donate
# reply body include:
# - endpoint: "donate"
# - enabled: bool
# - variant: "CRYPTO" or "LINK"
# - link: url
# - currencies: array with dict items
# - - example array: [ { "name": "BTC", "addr": "none" } ]
const DOMAIN := "denotatum.fun"
const PROTOCOL_PREFIX := "https://"
const SITE_URL := PROTOCOL_PREFIX + DOMAIN
const API_DOMAIN := "api." + DOMAIN
const REVIEW_GET_ENDPOINT := PROTOCOL_PREFIX + API_DOMAIN + "/reviews/review"
const REVIEW_POST_ENDPOINT := PROTOCOL_PREFIX + API_DOMAIN + "/reviews/postreview"
const REVIEW_LINK := SITE_URL + "/review.html"
const DONATE_ENDPOINT := PROTOCOL_PREFIX + API_DOMAIN + "/donate/donate"
const DONATE_LINK := SITE_URL + "/donate.html"

const _DEFAULT_LOCALE = "en"
const _JS_FINGERPRINT := """
try {
	let fp = get_fingerprint();
	if (fp.length === 0) {
		throw new Error("empty fp");
	}
	fp;
}
catch (e) { e.toString(); }
"""


func _ready() -> void:
	if OS.get_name() == "HTML5":
		use_threads = false
		JavaScript.eval(_read_js())


func http_request(url: String, data:=""):
	var lang := "Accept-Language: " + OS.get_locale_language()
	var agent_data := {
		"name": ProjectSettings.get_setting("application/config/name"),
		"version": str(Global.VERSION),
		"platform": OS.get_name(),
		"locale": TranslationServer.get_locale(),
		"id": "",
		"g_ver": Engine.get_version_info().string,
	}
	if agent_data.locale.empty():
		agent_data.locale = _DEFAULT_LOCALE
	if OS.get_name() == "HTML5":
		var fingerprint = JavaScript.eval(_JS_FINGERPRINT)
		if typeof(fingerprint) == TYPE_STRING:
			agent_data.id = fingerprint
	else:
		agent_data.id = OS.get_unique_id()
	var agent := "{name}/{version} ({platform}; {locale}; {id}) Godot/{g_ver}".format(
			agent_data
	)
	var headers = [
		lang,
		"User-Agent: " + agent,
		"Game-User-Agent: " + agent,
		"Content-Type: application/json"
	]
	if data.empty():
		return request(url, headers)
	else:
		return request(
				url,
				headers,
				true,
				HTTPClient.METHOD_POST,
				data
		)


func _read_js() -> String:
	var content := ""
	var file_fp = File.new()
	var file_lib = File.new()
	var error_fp = file_fp.open("res://global/js/fp.js", File.READ)
	var error_lib = file_lib.open("res://global/js/v3.js", File.READ)
	if error_fp == OK and error_lib == OK:
		var lib_content: String = Marshalls.utf8_to_base64(file_lib.get_as_text())
		content = file_fp.get_as_text() % lib_content
	file_fp.close()
	file_lib.close()
	return content
