extends Control


const _JS_EXIT := """
window.parent.postMessage("winclose", "*");
window.close();
"""

onready var _logo: Sprite = $Sprite
onready var _version: Label = $ScrollContainer/HBoxContainer/Version
onready var _donate_button: Button = $ScrollContainer/HBoxContainer/DonateButton
onready var _letter_button: Button = $ScrollContainer/HBoxContainer/LetterButton
onready var _review_button: Button = $ScrollContainer/HBoxContainer/ReviewButton


func _ready() -> void:
	get_tree().get_root().connect("size_changed", self, "_resize")
	_resize()
	_version.text = "v" + str(Global.VERSION)
	if Global.donate.can_show_donate_button():
		_donate_button.show()
		if Global.letter.can_show_letter_button():
			_letter_button.show()
	if Global.review.can_show_review_button():
		_review_button.show()
	Global.play_bg_music()
	get_tree().paused = false  # crutch for hearts


func _resize() -> void:
	_logo.position.x = get_viewport().size.x / 2
	if get_viewport().size.x < _logo.texture.get_size().x:
		var scale_val: float = get_viewport().size.x * 0.0025
		_logo.scale.x = scale_val
		_logo.scale.y = scale_val
	else:
		_logo.scale.x = 1
		_logo.scale.y = 1


func _on_Start_pressed() -> void:
	Global.stop_bg_music()
	if Global.back_scene:
		get_tree().paused = true
		Global.return_scene()
	else:
		Global.bg_goto_scene("res://game/game.tscn")


func _on_Settings_pressed() -> void:
	Global.goto_scene("res://menu/settings.tscn")


func _on_Exit_pressed() -> void:
	get_tree().quit(0)
	if OS.get_name() == "HTML5":
		JavaScript.eval(_JS_EXIT)


func _on_Statistic_pressed() -> void:
	Global.goto_scene("res://menu/statistic.tscn")


func _on_DonateButton_pressed() -> void:
	match(Global.donate.get_variant()):
		Global.donate.DonateVariant.CRYPTO:
			Global.goto_scene("res://menu/crypto_donate.tscn")
		Global.donate.DonateVariant.LINK:
			OS.shell_open(Global.donate.get_link())


func _on_ReviewButton_pressed() -> void:
	if OS.get_name() == "HTML5":
		OS.shell_open(Global.review.get_link())
	else:
		Global.goto_scene("res://menu/review.tscn")


func _on_About_pressed() -> void:
	Global.goto_scene("res://menu/about.tscn")


func _on_LetterButton_pressed() -> void:
	Global.goto_scene("res://menu/letter.tscn")
