extends Control


var _rating := 0.5

onready var _popup: PopupPanel = $CanvasLayer/PopupPanel
onready var _checkbox: CheckBox = $ScrollContainer/VBoxContainer/HBoxContainer/NeutralCheckBox
onready var _send_button: Button = $ScrollContainer/VBoxContainer/HBoxContainer2/SendButton
onready var _sent_label: Label = $CanvasLayer/PopupPanel/CenterContainer/SentLabel
onready var _error_label: Label = $CanvasLayer/PopupPanel/CenterContainer/ErrorLabel
onready var _username: LineEdit = $ScrollContainer/VBoxContainer/Username
onready var _text: TextEdit = $ScrollContainer/VBoxContainer/Text


func _ready() -> void:
	_checkbox.group.connect("pressed", self, "_rating_button_pressed")
	if Global.review.is_sent():
		_send_button.set_deferred("disabled", true)
	Global.review.connect("api_reply", self, "_api_reply")


func _exit_tree() -> void:
	Global.review.disconnect("api_reply", self, "_api_reply")


func _rating_button_pressed(button: CheckBox) -> void:
	if button.is_in_group("rating_bad"):
		_rating = 0.0
	elif button.is_in_group("rating_neutral"):
		_rating = 0.5
	elif button.is_in_group("rating_good"):
		_rating = 1.0


func _api_reply(data: Dictionary) -> void:
	if data.error:
		_send_button.set_deferred("disabled", false)
		_error_label.show()
		_sent_label.hide()
		_popup.show()
	else:
		_error_label.hide()
		_sent_label.show()
		_popup.show()


func _on_SendButton_pressed() -> void:
	var data := {
			"username": _username.text,
			"rating": _rating,
			"body": _text.text,
			"link": Global.review.get_link(),
	}
	Global.http_request(RestAPI.REVIEW_POST_ENDPOINT, to_json(data))
	_send_button.set_deferred("disabled", true)


func _on_BackButton_pressed() -> void:
	Global.goto_scene("res://menu/main_menu.tscn")


func _on_PopupPanel_mouse_exited() -> void:
	_popup.hide()


func _on_TextEdit_focus_entered() -> void:
	_popup.hide()
