extends Control


var _volume_master: float
var _volume_env: float
var _volume_music: float
var _volume_alerts: float
var _stars_move: bool
var _fragments: bool
var _light: bool
var _particles: bool
var _network_requests: bool
var _locale: String

onready var _slider_volume_master: HSlider = $ScrollContainer/GridContainer/VolumeMaster
onready var _slider_volume_env: HSlider = $ScrollContainer/GridContainer/VolumeEnv
onready var _slider_volume_music: HSlider = $ScrollContainer/GridContainer/VolumeMusic
onready var _slider_volume_alerts: HSlider = $ScrollContainer/GridContainer/VolumeAlerts
onready var _locale_option_button: OptionButton = $ScrollContainer/GridContainer/Locale
onready var _check_stars_move: CheckButton = $ScrollContainer/GridContainer/StarsMove
onready var _check_fragments: CheckButton = $ScrollContainer/GridContainer/Fragments
onready var _check_light: CheckButton = $ScrollContainer/GridContainer/Light
onready var _check_particles: CheckButton = $ScrollContainer/GridContainer/Particles
onready var _others_settings: Control = $CanvasLayer/OthersSettings
onready var _network_requests_button: CheckButton = (
		$CanvasLayer/OthersSettings/ScrollContainer/VBoxContainer/CenterContainer/VBoxContainer\
		/NetworkRequests
)
onready var _requests_description: RichTextLabel = (
		$CanvasLayer/OthersSettings/ScrollContainer/VBoxContainer/Description
)


func _ready() -> void:
	_fill_fields()
	_fill_gui()
	_requests_description.text = tr(_requests_description.text)


func _fill_fields() -> void:
	_volume_master = db2linear(Global.store.get_property(Global.Store.Property.VOLUME_MASTER))
	_volume_env = db2linear(Global.store.get_property(Global.Store.Property.VOLUME_ENV))
	_volume_music = db2linear(Global.store.get_property(Global.Store.Property.VOLUME_MUSIC))
	_volume_alerts = db2linear(Global.store.get_property(Global.Store.Property.VOLUME_ALERTS))
	_stars_move = Global.store.get_property(Global.Store.Property.STARS_MOVE)
	_fragments = Global.store.get_property(Global.Store.Property.FRAGMENTS_SPAWN)
	_light = Global.store.get_property(Global.Store.Property.LIGHT)
	_particles = Global.store.get_property(Global.Store.Property.PARTICLES)
	_network_requests = Global.store.get_property(Global.Store.Property.NETWORK_REQUESTS)
	_locale = Global.store.get_property(Global.Store.Property.LOCALE)


func _fill_gui() -> void:
	_slider_volume_master.value = _volume_master
	_slider_volume_env.value = _volume_env
	_slider_volume_music.value = _volume_music
	_slider_volume_alerts.value = _volume_alerts
	_check_stars_move.pressed = _stars_move
	_check_fragments.pressed = _fragments
	_check_light.pressed = _light
	_check_particles.pressed = _particles
	_network_requests_button.pressed = _network_requests
	_locale_option_button.clear()
	var locale_id := 0
	for locale_name in TranslationServer.get_loaded_locales():
		_locale_option_button.add_item(locale_name, locale_id)
		if locale_name == _locale:
			_locale_option_button.select(locale_id)
		locale_id += 1


func _save() -> void:
	Global.store.set_property(
			Global.Store.Property.VOLUME_MASTER, linear2db(_slider_volume_master.value)
	)
	Global.store.set_property(
			Global.Store.Property.VOLUME_ENV, linear2db(_slider_volume_env.value)
	)
	Global.store.set_property(
			Global.Store.Property.VOLUME_MUSIC, linear2db(_slider_volume_music.value)
	)
	Global.store.set_property(
			Global.Store.Property.VOLUME_ALERTS, linear2db(_slider_volume_alerts.value)
	)
	var locale_id: int = _locale_option_button.get_selected_id()
	if locale_id > -1:
		Global.store.set_property(
			Global.Store.Property.LOCALE,
			_locale_option_button.get_item_text(locale_id)
		)
	Global.store.set_property(Global.Store.Property.STARS_MOVE, _check_stars_move.pressed)
	Global.store.set_property(Global.Store.Property.FRAGMENTS_SPAWN, _check_fragments.pressed)
	Global.store.set_property(Global.Store.Property.LIGHT, _check_light.pressed)
	Global.store.set_property(Global.Store.Property.PARTICLES, _check_particles.pressed)
	Global.store.set_property(
			Global.Store.Property.NETWORK_REQUESTS, _network_requests_button.pressed
	)
	Global.store.save_settings()


func _on_SaveButton_pressed() -> void:
	_save()
	Global.goto_scene("res://menu/main_menu.tscn")


func _on_ResetButton_pressed() -> void:
	_fill_gui()


func _on_BackButton_pressed() -> void:
	_others_settings.hide()


func _on_OthersSettingsButton_pressed() -> void:
	_others_settings.show()
