extends GutTest


func test_bullets_use() -> void:
	var gun_vars = load("res://entity/gun/variants.gd")
	gun_vars = gun_vars.new()
	for bullet_key in Bullet.Variants.Variant:
		var bullet_val = Bullet.Variants.Variant[bullet_key]
		var found := false
		for gun_var in Gun.Variants.Variant.values():
			if gun_vars.get_vars(gun_var).bullets_variant == bullet_val:
				found = true
				break
		assert_true(found, bullet_key + " use")
	gun_vars.free()


func test_events_use() -> void:
	var events = load("res://game/gd/events.gd")
	events = events.new()
	var levels = load("res://game/gd/levels.gd")
	levels = levels.new()
	var list_events := []
	for level_val in Levels.Level.values():
		var vars = levels.get_level(level_val)
		for event_val in vars.events.values():
			list_events.append(event_val)
	for event_key in Events.EventVariant:
		var event_val = Events.EventVariant[event_key]
		assert_true(list_events.has(event_val), event_key + " use")
	events.free()
	levels.free()


func test_guns_use() -> void:
	var events = load("res://game/gd/events.gd")
	events = events.new()
	var levels = load("res://game/gd/levels.gd")
	levels = levels.new()
	var list_guns := []
	for level_val in Levels.Level.values():
		var vars: Dictionary = levels.get_level(level_val)
		list_guns.append(vars.gun_variant)
	for event_val in Events.EventVariant.values():
		var vars: Dictionary = events.get_vars(event_val)
		if vars.type != Events.EventType.GUN_UPGRADE:
			continue
		list_guns.append(vars.gun_variant)
	for gun_key in Gun.Variants.Variant:
		var gun_val = Gun.Variants.Variant[gun_key]
		assert_true(list_guns.has(gun_val), gun_key + " use")
	events.free()
	levels.free()


func test_meteors_use() -> void:
	var events = load("res://game/gd/events.gd")
	events = events.new()
	var levels = load("res://game/gd/levels.gd")
	levels = levels.new()
	var list_meteors := []
	for level_val in Levels.Level.values():
		var vars: Dictionary = levels.get_level(level_val)
		for meteor in vars.meteors:
			list_meteors.append(meteor.variant)
	for event_val in Events.EventVariant.values():
		var vars: Dictionary = events.get_vars(event_val)
		if vars.type != Events.EventType.METEORS_WAVE:
			continue
		for meteor in vars.meteors:
			list_meteors.append(meteor.variant)
	for meteor_key in Meteor.Variants.Variant:
		var meteor_val = Meteor.Variants.Variant[meteor_key]
		assert_true(list_meteors.has(meteor_val), meteor_key + " use")
	events.free()
	levels.free()


func test_levels_use() -> void:
	var levels = load("res://game/gd/levels.gd")
	levels = levels.new()
	var list_levels := []
	var exclude := [
		Levels.Level.TEST,
		Game.FIRST_LEVEL,
		Game.FIRST_ZEN_LEVEL,
		Game.ZEN_ENDLESS_LEVEL,
	]
	for level_val in Levels.Level.values():
		var vars: Dictionary = levels.get_level(level_val)
		list_levels.append(vars.next_level)
	for level_key in Levels.Level:
		var level_val = Levels.Level[level_key]
		if exclude.has(level_val):
			continue
		assert_true(list_levels.has(level_val), level_key + " use")
	levels.free()
